/*
 * Copyright (C) 2013 Martin Sebastian Wain
 * http://acidotunismo.com
 *
 * Released under CC-BY-3.0 license: http://creativecommons.org/licenses/by/3.0/
 * You can do derivative work and use it commercially as long as you credit my name properly.
 * 
 */

void setup() {
    Serial.begin(31250);
    Serial.println("MIDI IN DECODER");
}

const char *cmd_names[8] = {
    " OFF", "  ON", "AFTR", " CHG", "PROG", "CHAF", "PTCH", "SYS"
};
const char *sys_names[8] = {
    "  EX", " F/4", "SPOS", "SSEL", "0xF4", "0xF5", "TUNE", " EOX"
};

void loop() {
    if(Serial.available()) {
        byte type=Serial.read();
        byte cmd = type&0xf0;

        byte chan=type&0xf;
        byte p1,p2;
        if(cmd != 0xf0) { //System messages don't have a channel.
            Serial.print("CH-");
            Serial.print(chan, HEX);
        }
        Serial.print(' ');
        Serial.print(cmd_names[cmd>>4 & 0x7]);
        Serial.print(' ');
        switch(cmd) {
            //These ones have 2 parameters
            case 0x80:
            case 0x90:
            case 0xa0:
            case 0xb0:
            case 0xe0:
                while(Serial.available()<2);       //Just wait for another two incoming parameters
                Serial.print(p1, HEX);
                Serial.write(' ');
                Serial.print(p2, HEX);
                p1 = Serial.read();
                p2 = Serial.read();
                break;
            //These ones have only one parameter
            case 0xc0:
            case 0xd0:
                while(!Serial.available());        //Just wait for another incoming parameter
                p1 = Serial.read();
                Serial.print(p1, HEX);
                break;
            //Handle system ones specially
            case 0xf0:
                Serial.print(sys_names[type&0x7]);
                Serial.print(' ');
                switch(type) {
                   //SysEx messages span until a 0xf7
                   case 0xf0:
                        while(Serial.available()) {
                            p1 = Serial.read();
                            Serial.print(p1, HEX);
                            Serial.print(' ');
                            if(p1 == 0xf7) break;
                        }
                        break;
                    case 0xf1:
                    case 0xf3:
                        while(!Serial.available());        //Just wait for another incoming parameter
                        p1 = Serial.read();
                        Serial.print(p1, HEX);
                        break;
                    case 0xf2:
                        while(Serial.available()<2);       //Just wait for another two incoming parameters
                        Serial.print(p1, HEX);
                        Serial.write(' ');
                        Serial.print(p2, HEX);
                        p1 = Serial.read();
                        p2 = Serial.read();
                        break;
                }
                break;
        }
        Serial.print("\n");
    }

}
